(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rxjs/Subject'), require('@angular/common/http'), require('rxjs/observable/of'), require('rxjs/observable/throw'), require('rxjs/operators')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', 'rxjs/Subject', '@angular/common/http', 'rxjs/observable/of', 'rxjs/observable/throw', 'rxjs/operators'], factory) :
	(factory((global['ngx-http-retry'] = {}),global.ng.core,global.Rx,global.ng.common.http,global.Rx.Observable,global.Rx.Observable,global.Rx.Observable.prototype));
}(this, (function (exports,core,Subject,http,of,_throw,operators) { 'use strict';

var HTTP_REQUEST_RETRY_STRATEGIES = new core.InjectionToken('HTTP_REQUEST_RETRY_STRATEGIES');
var httpRetryFailuresSubject = new Subject.Subject();
var NgxHttpRetryService = /** @class */ (function () {
    function NgxHttpRetryService() {
        this.httpRetryFailures = httpRetryFailuresSubject.asObservable();
    }
    return NgxHttpRetryService;
}());
NgxHttpRetryService.decorators = [
    { type: core.Injectable },
];
NgxHttpRetryService.ctorParameters = function () { return []; };
function httpRequestRetry(retryStrategies) {
    var counts = {};
    function retryWhenNotifier(errors) {
        return errors.pipe(operators.switchMap(function (error) {
            if (error instanceof http.HttpErrorResponse) {
                var retryStrategyIndex = retryStrategies.findIndex(function (rs) { return rs.statuses.includes(error.status); });
                var retryStrategy = retryStrategyIndex > -1 ? retryStrategies[retryStrategyIndex] : undefined;
                counts[retryStrategyIndex] = (counts[retryStrategyIndex] || 0) + 1;
                var shouldRetry = retryStrategy && counts[retryStrategyIndex] < retryStrategy.maxCount;
                var retryDelay = retryStrategy && retryStrategy.delayFn ? retryStrategy.delayFn(counts[retryStrategyIndex]) : 0;
                if (!shouldRetry && retryStrategy) {
                    httpRetryFailuresSubject.next(error);
                    if (retryStrategy.onFailure) {
                        retryStrategy.onFailure(error);
                    }
                }
                return shouldRetry ? of.of(error).pipe(operators.delay(retryDelay)) : _throw._throw(error);
            }
        }));
    }
    return function (source) { return source.pipe(operators.retryWhen(retryWhenNotifier)); };
}
var attemptNumberHeader = 'X-Request-Attempt-Number';
var NgxHttpRetryInterceptor = /** @class */ (function () {
    function NgxHttpRetryInterceptor(retryStrategies) {
        this.retryStrategies = retryStrategies;
    }
    NgxHttpRetryInterceptor.prototype.intercept = function (request, next) {
        var result;
        if (this.retryStrategies.length && request.method.toUpperCase() === 'GET') {
            var attemptNumber_1 = 0;
            result = of.of(undefined).pipe(operators.tap(function () {
                attemptNumber_1++;
            }), operators.switchMap(function () { return next.handle(getRequestWithAttemptNumber(request, attemptNumber_1)); }), httpRequestRetry(this.retryStrategies));
        }
        else {
            result = next.handle(request);
        }
        return result;
    };
    return NgxHttpRetryInterceptor;
}());
NgxHttpRetryInterceptor.decorators = [
    { type: core.Injectable },
];
NgxHttpRetryInterceptor.ctorParameters = function () { return [
    { type: Array, decorators: [{ type: core.Inject, args: [HTTP_REQUEST_RETRY_STRATEGIES,] },] },
]; };
var ngxHttpRetryInterceptorProvider = {
    provide: http.HTTP_INTERCEPTORS,
    useClass: NgxHttpRetryInterceptor,
    multi: true
};
function getRequestWithAttemptNumber(request, attemptNumber) {
    return request.clone({ headers: request.headers.append(attemptNumberHeader, attemptNumber.toString()) });
}
var NgxHttpRetryModule = /** @class */ (function () {
    function NgxHttpRetryModule(parentModule) {
        if (parentModule) {
            throw new Error('NgxHttpRetryModule is already loaded. Import it only once (e.g. in your AppModule or CoreModule).');
        }
    }
    NgxHttpRetryModule.forRoot = function () {
        return {
            ngModule: NgxHttpRetryModule,
            providers: [NgxHttpRetryService, ngxHttpRetryInterceptorProvider]
        };
    };
    return NgxHttpRetryModule;
}());
NgxHttpRetryModule.decorators = [
    { type: core.NgModule, args: [{
                imports: [http.HttpClientModule]
            },] },
];
NgxHttpRetryModule.ctorParameters = function () { return [
    { type: NgxHttpRetryModule, decorators: [{ type: core.Optional }, { type: core.SkipSelf },] },
]; };

exports.HTTP_REQUEST_RETRY_STRATEGIES = HTTP_REQUEST_RETRY_STRATEGIES;
exports.httpRequestRetry = httpRequestRetry;
exports.NgxHttpRetryService = NgxHttpRetryService;
exports.NgxHttpRetryModule = NgxHttpRetryModule;
exports.ɵa = NgxHttpRetryInterceptor;
exports.ɵb = ngxHttpRetryInterceptorProvider;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-http-retry.umd.js.map
