import { InjectionToken, Injectable, Inject, NgModule, Optional, SkipSelf } from '@angular/core';
import { Subject } from 'rxjs/Subject';
import { HttpErrorResponse, HTTP_INTERCEPTORS, HttpClientModule } from '@angular/common/http';
import { of } from 'rxjs/observable/of';
import { _throw } from 'rxjs/observable/throw';
import { delay, retryWhen, switchMap, tap } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const HTTP_REQUEST_RETRY_STRATEGIES = new InjectionToken('HTTP_REQUEST_RETRY_STRATEGIES');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const httpRetryFailuresSubject = new Subject();
class NgxHttpRetryService {
    constructor() {
        this.httpRetryFailures = httpRetryFailuresSubject.asObservable();
    }
}
NgxHttpRetryService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
NgxHttpRetryService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

/**
 * @param {?} retryStrategies
 * @return {?}
 */
function httpRequestRetry(retryStrategies) {
    const /** @type {?} */ counts = {};
    /**
     * @param {?} errors
     * @return {?}
     */
    function retryWhenNotifier(errors) {
        return errors.pipe(switchMap(error => {
            if (error instanceof HttpErrorResponse) {
                const /** @type {?} */ retryStrategyIndex = retryStrategies.findIndex(rs => rs.statuses.includes(error.status));
                const /** @type {?} */ retryStrategy = retryStrategyIndex > -1 ? retryStrategies[retryStrategyIndex] : undefined;
                counts[retryStrategyIndex] = (counts[retryStrategyIndex] || 0) + 1;
                const /** @type {?} */ shouldRetry = retryStrategy && counts[retryStrategyIndex] < retryStrategy.maxCount;
                const /** @type {?} */ retryDelay = retryStrategy && retryStrategy.delayFn ? retryStrategy.delayFn(counts[retryStrategyIndex]) : 0;
                if (!shouldRetry && retryStrategy) {
                    httpRetryFailuresSubject.next(error);
                    if (retryStrategy.onFailure) {
                        retryStrategy.onFailure(error);
                    }
                }
                return shouldRetry ? of(error).pipe(delay(retryDelay)) : _throw(error);
            }
        }));
    }
    return (source) => source.pipe(retryWhen(retryWhenNotifier));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const attemptNumberHeader = 'X-Request-Attempt-Number';
class NgxHttpRetryInterceptor {
    /**
     * @param {?} retryStrategies
     */
    constructor(retryStrategies) {
        this.retryStrategies = retryStrategies;
    }
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    intercept(request, next) {
        let /** @type {?} */ result;
        if (this.retryStrategies.length && request.method.toUpperCase() === 'GET') {
            let /** @type {?} */ attemptNumber = 0;
            result = of(undefined).pipe(tap(() => {
                attemptNumber++;
            }), switchMap(() => next.handle(getRequestWithAttemptNumber(request, attemptNumber))), httpRequestRetry(this.retryStrategies));
        }
        else {
            result = next.handle(request);
        }
        return result;
    }
}
NgxHttpRetryInterceptor.decorators = [
    { type: Injectable },
];
/** @nocollapse */
NgxHttpRetryInterceptor.ctorParameters = () => [
    { type: Array, decorators: [{ type: Inject, args: [HTTP_REQUEST_RETRY_STRATEGIES,] },] },
];
const ngxHttpRetryInterceptorProvider = {
    provide: HTTP_INTERCEPTORS,
    useClass: NgxHttpRetryInterceptor,
    multi: true
};
/**
 * @param {?} request
 * @param {?} attemptNumber
 * @return {?}
 */
function getRequestWithAttemptNumber(request, attemptNumber) {
    return request.clone({ headers: request.headers.append(attemptNumberHeader, attemptNumber.toString()) });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class NgxHttpRetryModule {
    /**
     * @param {?} parentModule
     */
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('NgxHttpRetryModule is already loaded. Import it only once (e.g. in your AppModule or CoreModule).');
        }
    }
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: NgxHttpRetryModule,
            providers: [NgxHttpRetryService, ngxHttpRetryInterceptorProvider]
        };
    }
}
NgxHttpRetryModule.decorators = [
    { type: NgModule, args: [{
                imports: [HttpClientModule]
            },] },
];
/** @nocollapse */
NgxHttpRetryModule.ctorParameters = () => [
    { type: NgxHttpRetryModule, decorators: [{ type: Optional }, { type: SkipSelf },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { HTTP_REQUEST_RETRY_STRATEGIES, httpRequestRetry, NgxHttpRetryService, NgxHttpRetryModule, NgxHttpRetryInterceptor as ɵa, ngxHttpRetryInterceptorProvider as ɵb };
//# sourceMappingURL=ngx-http-retry.js.map
